@@ -15,10 +15,10 @@ import (
 	"strings"
 
 	"github.com/josephburnett/jd/v2"
-	"github.com/josephburnett/jd/v2/web/serve"
+	"github.com/josephburnett/jd/v2/internal/web/serve"
 )
 
-const version = "2.3.1"
+const version = "2.4.0"
 
 var (
 	color         = flag.Bool("color", false, "Print color diff")
@@ -166,7 +166,9 @@ func parseOptions() ([]jd.Option, error) {
 	if *format == "merge" {
 		options = append(options, jd.MERGE)
 	}
-	options = append(options, jd.Precision(*precision))
+	if *precision != 0.0 {
+		options = append(options, jd.Precision(*precision))
+	}
 	return options, nil
 }
 
@@ -180,12 +182,19 @@ func printUsageAndExit() {
 		`When FILE2 is omitted the second input is read from STDIN.`,
 		`When patching (-p) FILE1 is a diff.`,
 		``,
+		`Options Header: When options are provided, they are displayed at the`,
+		`beginning of the diff output to show how the diff was produced:`,
+		`  ^ "SET"                     (arrays treated as sets)`,
+		`  ^ {"precision":0.001}       (number comparison precision)`,
+		`  ^ {"setkeys":["id"]}        (object matching keys)`,
+		`This helps understand diff behavior and enables reproducible results.`,
+		``,
 		`Options:`,
 		`  -color       Print color diff.`,
 		`  -p           Apply patch FILE1 to FILE2 or STDIN.`,
 		`  -o=FILE3     Write to FILE3 instead of STDOUT.`,
 		`  -opts='[]'   JSON array of options. Supports global options and PathOptions.`,
-		`               Global: ["SET"], ["MULTISET"], [{"precision":0.1}], [{"setkeys":["id"]}]`,
+		`               Global: ["SET"], ["MULTISET"], [{"precision":0.1}], [{"setkeys":["id"]}], ["DIFF_ON"], ["DIFF_OFF"]`,
 		`               PathOptions target specific paths: [{"@":["path"],"^":["SET"]}]`,
 		`               Example: [{"@":["users"],"^":["SET"]},{"@":["scores",0],"^":[{"precision":0.1}]}]`,
 		`  -set         Treat arrays as sets. Same as -opts='["SET"]'.`,
@@ -211,6 +220,8 @@ func printUsageAndExit() {
 		`  jd -f merge a.json b.json`,
 		`  jd -opts='[{"@":["items"],"^":["SET"]}]' a.json b.json`,
 		`  jd -opts='[{"@":["temperature"],"^":[{"precision":0.1}]}]' a.json b.json`,
+		`  jd -opts='[{"@":["timestamp"],"^":["DIFF_OFF"]}]' a.json b.json`,
+		`  jd -opts='[{"@":[],"^":["DIFF_OFF"]},{"@":["userdata"],"^":["DIFF_ON"]}]' a.json b.json`,
 		``,
 		`Version: ` + version,
 		``,
@@ -272,6 +283,8 @@ func diff(a, b string, options []jd.Option) (string, bool, error) {
 	}
 	diff := aNode.Diff(bNode, options...)
 	var renderOptions []jd.Option
+	// Include all the original options to show in the header
+	renderOptions = append(renderOptions, options...)
 	if *color {
 		renderOptions = append(renderOptions, jd.COLOR)
 	}
