@@ -34,7 +34,7 @@ func (n jsonNull) hashCode(_ *options) [8]byte {
 }
 
 func (n jsonNull) Diff(node JsonNode, opts ...Option) Diff {
-	o := refine(&options{retain: opts}, nil)
+	o := refine(newOptions(opts), nil)
 	return n.diff(node, make(Path, 0), o, getPatchStrategy(o))
 }
 
@@ -44,7 +44,10 @@ func (n jsonNull) diff(
 	opts *options,
 	strategy patchStrategy,
 ) Diff {
-	return diff(n, node, path, opts, strategy)
+	// Use event-driven diff architecture
+	events := generateSimpleEvents(n, node, opts)
+	processor := newSimpleDiffProcessor(path, opts, strategy)
+	return processor.ProcessEvents(events)
 }
 
 func (n jsonNull) Patch(d Diff) (JsonNode, error) {
