@@ -68,7 +68,7 @@ func (s jsonSet) hashCode(opts *options) [8]byte {
 }
 
 func (s jsonSet) Diff(j JsonNode, opts ...Option) Diff {
-	o := refine(&options{retain: opts}, nil)
+	o := refine(newOptions(opts), nil)
 	return s.diff(j, make(Path, 0), o, getPatchStrategy(o))
 }
 
@@ -78,106 +78,25 @@ func (s1 jsonSet) diff(
 	opts *options,
 	strategy patchStrategy,
 ) Diff {
-	d := make(Diff, 0)
 	s2, ok := n.(jsonSet)
 	if !ok {
-		// Different types
-		var e DiffElement
-		switch strategy {
-		case mergePatchStrategy:
-			e = DiffElement{
-				Metadata: Metadata{
-					Merge: true,
-				},
-				Path: path.clone(),
-				Add:  nodeList(n),
-			}
-		default:
-			e = DiffElement{
-				Path:   path.clone(),
-				Remove: nodeList(s1),
-				Add:    nodeList(n),
-			}
-		}
-		return append(d, e)
+		// Different types - use simple replace event
+		events := generateSimpleEvents(s1, n, opts)
+		processor := newsetDiffProcessor(path, opts, strategy)
+		return processor.ProcessEvents(events)
 	}
+
+	// Handle merge patch strategy for same types
 	if strategy == mergePatchStrategy && !s1.Equals(n) {
-		e := DiffElement{
-			Metadata: Metadata{
-				Merge: true,
-			},
-			Path: path.clone(),
-			Add:  nodeList(n),
-		}
-		return append(d, e)
+		events := generateSimpleEvents(s1, n, opts)
+		processor := newsetDiffProcessor(path, opts, strategy)
+		return processor.ProcessEvents(events)
 	}
-	s1Map := make(map[[8]byte]JsonNode)
-	for _, v := range s1 {
-		var hc [8]byte
-		if o, ok := v.(jsonObject); ok {
-			// Hash objects by their identity.
-			hc = o.ident(opts)
-		} else {
-			// Everything else by full content.
-			hc = v.hashCode(opts)
-		}
-		s1Map[hc] = v
-	}
-	s2Map := make(map[[8]byte]JsonNode)
-	for _, v := range s2 {
-		var hc [8]byte
-		if o, ok := v.(jsonObject); ok {
-			// Hash objects by their identity.
-			hc = o.ident(opts)
-		} else {
-			// Everything else by full content.
-			hc = v.hashCode(opts)
-		}
-		s2Map[hc] = v
-	}
-	s1Hashes := make(hashCodes, 0)
-	for hc := range s1Map {
-		s1Hashes = append(s1Hashes, hc)
-	}
-	sort.Sort(s1Hashes)
-	s2Hashes := make(hashCodes, 0)
-	for hc := range s2Map {
-		s2Hashes = append(s2Hashes, hc)
-	}
-	sort.Sort(s2Hashes)
-	e := DiffElement{
-		Path:   append(path.clone(), PathSet{}),
-		Remove: nodeList(),
-		Add:    nodeList(),
-	}
-	for _, hc := range s1Hashes {
-		n2, ok := s2Map[hc]
-		if !ok {
-			// Deleted value.
-			e.Remove = append(e.Remove, s1Map[hc])
-		} else {
-			// Changed value.
-			o1, isObject1 := s1Map[hc].(jsonObject)
-			o2, isObject2 := n2.(jsonObject)
-			if isObject1 && isObject2 {
-				// Sub diff objects with same identity.
-				p := append(path.clone(), newPathSetKeys(o1, opts))
-				subDiff := o1.diff(o2, p, opts, strategy)
-				d = append(d, subDiff...)
-			}
-		}
-	}
-	for _, hc := range s2Hashes {
-		_, ok := s1Map[hc]
-		if !ok {
-			// Added value.
-			e.Add = append(e.Add, s2Map[hc])
-		}
-	}
-	if len(e.Remove) > 0 || len(e.Add) > 0 {
-		d = append(d, e)
-	}
-	return d
+
+	// Same type - use set-specific event generation
+	events := generateSetdiffEvents(s1, s2, opts)
+	processor := newsetDiffProcessor(path, opts, strategy)
+	return processor.ProcessEvents(events)
 }
 
 func (s jsonSet) Patch(d Diff) (JsonNode, error) {
@@ -290,3 +209,213 @@ func (s jsonSet) patch(
 	}
 	return newValue, nil
 }
+
+// ============================================================================
+// SET-SPECIFIC EVENT-DRIVEN DIFF SYSTEM
+// ============================================================================
+
+// setElementEvent represents operations on set elements
+type setElementEvent struct {
+	Operation string // "ADD" or "REMOVE"
+	Element   JsonNode
+	Hash      [8]byte // For identity tracking
+}
+
+func (e setElementEvent) String() string {
+	return fmt.Sprintf("SET_%s(%s)", e.Operation, e.Element.Json())
+}
+
+func (e setElementEvent) GetType() string { return "SET_ELEMENT" }
+
+// setObjectDiffEvent represents an object in a set that needs recursive diffing
+type setObjectDiffEvent struct {
+	OldObject JsonNode
+	NewObject JsonNode
+	Hash      [8]byte // Identity hash
+}
+
+func (e setObjectDiffEvent) String() string {
+	return fmt.Sprintf("SET_OBJECT_DIFF(%s -> %s)", e.OldObject.Json(), e.NewObject.Json())
+}
+
+func (e setObjectDiffEvent) GetType() string { return "SET_OBJECT_DIFF" }
+
+// setDiffProcessor processes set diff events
+type setDiffProcessor struct {
+	*baseDiffProcessor
+}
+
+func newsetDiffProcessor(path Path, opts *options, strategy patchStrategy) *setDiffProcessor {
+	return &setDiffProcessor{
+		baseDiffProcessor: newBaseDiffProcessor(path, opts, strategy),
+	}
+}
+
+func (p *setDiffProcessor) ProcessEvents(events []diffEvent) Diff {
+	p.debugLog("Starting to process %d set events", len(events))
+
+	// Collect all add/remove events for the set operation
+	var setElement *DiffElement
+
+	for i, event := range events {
+		p.debugLog("Processing event %d: %s", i, event.String())
+
+		switch e := event.(type) {
+		case setElementEvent:
+			if setElement == nil {
+				setElement = &DiffElement{
+					Path:   append(p.path.clone(), PathSet{}),
+					Remove: []JsonNode{},
+					Add:    []JsonNode{},
+				}
+			}
+
+			if e.Operation == "REMOVE" {
+				setElement.Remove = append(setElement.Remove, e.Element)
+			} else if e.Operation == "ADD" {
+				setElement.Add = append(setElement.Add, e.Element)
+			}
+
+		case setObjectDiffEvent:
+			p.processsetObjectDiffEvent(e)
+
+		case simpleReplaceEvent:
+			p.processsimpleReplaceEvent(e)
+
+		default:
+			p.debugLog("WARNING: Unknown event type for setDiffProcessor: %T", event)
+		}
+	}
+
+	// Add the accumulated set diff element if it has changes
+	if setElement != nil && (len(setElement.Remove) > 0 || len(setElement.Add) > 0) {
+		p.finalDiff = append(p.finalDiff, *setElement)
+	}
+
+	p.debugLog("Final diff has %d elements", len(p.finalDiff))
+	return p.finalDiff
+}
+
+func (p *setDiffProcessor) processsetObjectDiffEvent(event setObjectDiffEvent) {
+	p.debugLog("Processing set object diff: %s -> %s", event.OldObject.Json(), event.NewObject.Json())
+
+	// For set object diffs, we need to create a path with PathSetKeys
+	o1, _ := event.OldObject.(jsonObject)
+	setKeysPath := newPathSetKeys(o1, p.opts)
+	subPath := append(p.path.clone(), setKeysPath)
+
+	subDiff := event.OldObject.diff(event.NewObject, subPath, p.opts, p.strategy)
+	p.finalDiff = append(p.finalDiff, subDiff...)
+}
+
+func (p *setDiffProcessor) processsimpleReplaceEvent(event simpleReplaceEvent) {
+	p.debugLog("Processing simple replace: %s -> %s", event.OldValue.Json(), event.NewValue.Json())
+
+	var e DiffElement
+	switch p.strategy {
+	case mergePatchStrategy:
+		e = DiffElement{
+			Metadata: Metadata{
+				Merge: true,
+			},
+			Path: p.path.clone(),
+			Add:  []JsonNode{event.NewValue},
+		}
+	default:
+		e = DiffElement{
+			Path:   p.path.clone(),
+			Remove: []JsonNode{event.OldValue},
+			Add:    []JsonNode{event.NewValue},
+		}
+	}
+
+	p.finalDiff = append(p.finalDiff, e)
+}
+
+// generateSetdiffEvents analyzes two sets and generates appropriate diff events
+func generateSetdiffEvents(s1, s2 jsonSet, opts *options) []diffEvent {
+	if !opts.diffingOn {
+		return []diffEvent{} // No events when diffing is off
+	}
+
+	var events []diffEvent
+
+	// Create hash maps for identity-based comparison
+	s1Map := make(map[[8]byte]JsonNode)
+	for _, v := range s1 {
+		var hc [8]byte
+		if o, ok := v.(jsonObject); ok {
+			// Hash objects by their identity
+			hc = o.ident(opts)
+		} else {
+			// Everything else by full content
+			hc = v.hashCode(opts)
+		}
+		s1Map[hc] = v
+	}
+
+	s2Map := make(map[[8]byte]JsonNode)
+	for _, v := range s2 {
+		var hc [8]byte
+		if o, ok := v.(jsonObject); ok {
+			// Hash objects by their identity
+			hc = o.ident(opts)
+		} else {
+			// Everything else by full content
+			hc = v.hashCode(opts)
+		}
+		s2Map[hc] = v
+	}
+
+	// Get sorted hash codes for deterministic ordering
+	s1Hashes := make(hashCodes, 0, len(s1Map))
+	for hc := range s1Map {
+		s1Hashes = append(s1Hashes, hc)
+	}
+	sort.Sort(s1Hashes)
+
+	s2Hashes := make(hashCodes, 0, len(s2Map))
+	for hc := range s2Map {
+		s2Hashes = append(s2Hashes, hc)
+	}
+	sort.Sort(s2Hashes)
+
+	// Process removes first (sorted by hash)
+	for _, hc := range s1Hashes {
+		v1 := s1Map[hc]
+		if v2, ok := s2Map[hc]; !ok {
+			// Deleted value
+			events = append(events, setElementEvent{
+				Operation: "REMOVE",
+				Element:   v1,
+				Hash:      hc,
+			})
+		} else {
+			// Check for object diffs with same identity
+			o1, isObject1 := v1.(jsonObject)
+			o2, isObject2 := v2.(jsonObject)
+			if isObject1 && isObject2 && !o1.equals(o2, opts) {
+				// Sub diff objects with same identity
+				events = append(events, setObjectDiffEvent{
+					OldObject: o1,
+					NewObject: o2,
+					Hash:      hc,
+				})
+			}
+		}
+	}
+
+	// Process adds (sorted by hash)
+	for _, hc := range s2Hashes {
+		if _, ok := s1Map[hc]; !ok {
+			// Added value
+			events = append(events, setElementEvent{
+				Operation: "ADD",
+				Element:   s2Map[hc],
+				Hash:      hc,
+			})
+		}
+	}
+
+	return events
+}
