@@ -34,7 +34,7 @@ func (s jsonString) hashCode(_ *options) [8]byte {
 }
 
 func (s jsonString) Diff(n JsonNode, opts ...Option) Diff {
-	o := refine(&options{retain: opts}, nil)
+	o := refine(newOptions(opts), nil)
 	return s.diff(n, make(Path, 0), o, getPatchStrategy(o))
 }
 
@@ -44,7 +44,10 @@ func (s1 jsonString) diff(
 	opts *options,
 	strategy patchStrategy,
 ) Diff {
-	return diff(s1, n, path, opts, strategy)
+	// Use event-driven diff architecture
+	events := generateSimpleEvents(s1, n, opts)
+	processor := newSimpleDiffProcessor(path, opts, strategy)
+	return processor.ProcessEvents(events)
 }
 
 func (s jsonString) Patch(d Diff) (JsonNode, error) {
